/** Class that describes a circular DNA sequence.  Uses BasePair
 *  together with DLList to manage the data.  Most of the methods 
 *  are wrappers for functionality provided by DLList.
 *  @author mwa
 */
public class DNA{
 
    private DLList<BasePair> sequence;  // our DNA sequence

    /** Constructs a DNA sequence using a string of bases.
     *  If there are invalid bases the character is ignored
     *  and a warning is issued to the console.
     *  @param string of bases
     */
    public DNA(String pairs){
	sequence = new DLList();
	for (int i = 0; i < pairs.length(); i++){
	    BasePair bp = BasePair.getBasePair(pairs.charAt(i));
	    if (bp == null)
		System.err.println("Warning -- invalid character in the sequence, ignoring");
	    sequence.insert(bp);
	    sequence.translate(1);
	}
    }

    /** Shifts the sequence in the current direction.
     *  @param amount to shift by
     */
    public void translate(int dir){
	sequence.translate(dir);
    }

    /** Removes the element in focus 
     */
    public void remove(){
	sequence.remove();
    }

    /** Removes the next count elements after and including
     *  the focus
     *  @param the number to remove
     */
    public void remove(int count){

	while(count > 0 && sequence.remove()){
	    count--;
	}

    }

    /** Reverses the sequence (it doesn't really change anything
     *  just the way that DLList accesses the elements).
     */
    public void reverse(){
	sequence.reverse();
    }

    /** Complements every base in the sequence 
     */
    public void invert(){
	for (int i = 0; i < sequence.getSize(); i++){
	    sequence.setCurrElement(sequence.getCurrElement().complement());
	    sequence.translate(1);
	}
    }

    /** Inserts the arguments immediately before the focus.
     *  @param array of BasePairs to insert
     */
    public void insert(BasePair[] bps){
	for (int i = 0; i < bps.length; i++){
	    sequence.insert(bps[i]);
	    sequence.translate(1);
	}
	sequence.translate(-bps.length); // set the focus
    }

    /** Searches for the genes between start and end codons
     *  see the assignment description for further information
     *  @param start codon array
     *  @param end codon array
     *  @return the string of genes
     */
    public String transcribe(BasePair[] start, BasePair[] end){
	String ret = "";

	if (sequence.isEmpty()) return ret;

	BasePair[] codon = new BasePair[3];

	do{
	    codon[0] = sequence.getCurrElement();   sequence.translate(1);
	    codon[1] = sequence.getCurrElement();   sequence.translate(1);
	    codon[2] = sequence.getCurrElement();   sequence.translate(1);
	} while ((!codon[0].equals(start[0]) 
		  || !codon[1].equals(start[1])
		  || !codon[2].equals(start[2])));

	while(true){
	    
	    codon[0] = sequence.getCurrElement();   sequence.translate(1);
	    codon[1] = sequence.getCurrElement();   sequence.translate(1);
	    codon[2] = sequence.getCurrElement();   sequence.translate(1);

	    if (codon[0].equals(end[0]) && codon[1].equals(end[1])
		&& codon[2].equals(end[2]))
		break;
	    else 
		ret += "" + codon[0] + codon[1] + codon[2];
	}

	return ret;
    }

    /** Returns a String representing the base pairs in the sequence
     *  at most chars characters are allowed, if chars is greater than
     *  the length of the sequence the length of the sequence is used
     *  as the limit instead.
     *  @param number of bases to include
     *  @param String containing the bases
     */
    public String formString(int chars){
	String ret = "";
	
	for (int i = 0; i < Math.min(chars,sequence.getSize()); i++){
	    ret += sequence.getCurrElement();
	    sequence.translate(1);
	}
	sequence.translate(-Math.min(chars,sequence.getSize()));

	return ret;
    }





}
