import java.util.Scanner;
import java.io.*;

/** This is the application class for the Genome assignment.
 *  The main purpose of this class to handle program control 
 *  flow. It also handles console and file I/O.
 *  All of the operations on the DNA are handled by the DNA class.
 *  @author mwa.
 */
public class Genome{

    private static Scanner stdin = new Scanner(System.in);

    /** Reads in a codon from the console
     *  repeats prompt until a valid codon is entered
     *  @return an array of 3 BasePairs
     */
    private static BasePair[] getInputSequence(){
	start:
	while(true){
	    System.out.println("Please enter a sequence:");
	    
	    String input = stdin.nextLine();
	    
	    // validate length
	    if (input.length() != 3){
		System.err.println("Invalid input - the sequence must be exactly 3 base pairs");
		continue start;
	    }

	    // validate contents
	    BasePair[] bps = new BasePair[3];	    
	    for (int i = 0; i < input.length(); i++){
		bps[i] = BasePair.getBasePair(input.charAt(i));  // this returns null if invalid
		if (bps[i] == null){
		    System.err.println("Invalid input - '"+input.charAt(i)+"'");
		    continue start;
		}
	    }
	    return bps;
	}

    }

    /** Prompt a user to input an int, reprompts if the number is not positive
     *  @param the prompt to use
     *  @return the value the user entered
     */
    private static int getInt(String prompt){
	int choice = -1;
	while (true){
	    System.out.println(prompt);
	    String input = stdin.nextLine();

	    try{
		choice = Integer.parseInt(input);
		if (choice <= 0) continue;
		return choice;
	    } catch (Exception e){}
	    System.out.print("Please enter a positive number: ");
	}
	
    }

    /** Displays the program menu and return the value the user entered
     *  the value will be a number between 1 and 6.
     *  @return the choice the user entered.
     */
    private static int displayMenu(){
	
	System.out.print("Genome Options: \n" +
			 "1. Reverse and complement sequence \n" +
			 "2. Insert base pairs \n" +
			 "3. Remove base pairs \n" +
			 "4. Shift \n"+
			 "5. Search for gene\n" +
			 "6. Quit \n"+
			 "Enter choice (1-6): ");
	
	int choice = -1;
	while(true){
	    String input = stdin.nextLine();
	    try{
		choice = Integer.parseInt(input);
		if (choice < 1 || choice > 6) continue;
		break;
	    } catch (Exception e){}
	    System.out.print("Invalid choice, please choose a number (1-6): ");
	}
	return choice;
    }

    /** Main method 
     *  - reads file from commandline
     *  - loops over menu and operations
     */
    public static void main(String[] args){
	
	// Check the user called our program correctly
	if (args.length != 1){
	    System.err.println("usage: java Genome <dna_filename>");
	    return;
	}

	// Read in the DNA from the file
	DNA dna = null;
	try{
	    Scanner s = new Scanner(new File(args[0]));
	    String line = s.nextLine();
	    dna = new DNA(line);  // parse the DNA and create object
	} catch (IOException e){
	    System.err.println("Error reading file");
	    return;
	}

	// Main loop
	while(true){
	    // Display the sequence
	    System.out.println("Sequence: \n");
	    System.out.println("\t" + dna.formString(20) + "\n");

	    // Display the menu
	    int choice = displayMenu();

	    // Choose the appropriate operation
	    switch(choice){
	    case 1:
		// Reverse and complement
		dna.reverse();
		dna.invert();
		break;
	    case 2:
		// Insert base pairs
		dna.insert(getInputSequence());
		break;
	    case 3:
		// Remove base pairs
		dna.remove(getInt("Please enter the number of base pairs to delete: "));
		break;
	    case 4:
		// Shift base pairs
		dna.translate(getInt("Please enter the number of base pairs to shift by: "));
		break;
	    case 5:
		// Search for gene
		System.out.println("Start Codon:");
		BasePair[] start = getInputSequence();
		System.out.println("End Codon:");
		BasePair[] end = getInputSequence();
		System.out.println("Result of search: "+dna.transcribe(start,end));
		break;
	    case 6:
		// Quit
		return;
	    default: break;
	    }
	}




    }











}
